@@ -1,5 +1,5 @@
 /* cksum -- calculate and print POSIX.2 checksums and sizes of files
-   Copyright (C) 1992, 1995 Free Software Foundation, Inc.
+   Copyright (C) 92, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -85,7 +85,7 @@ remainder (m)
   return rem & 0xFFFFFFFF;	/* Make it run on 64-bit machine.  */
 }
 
-void
+int
 main ()
 {
   int i;
@@ -99,7 +99,7 @@ main ()
 	      remainder (i * 5 + 4), remainder (i * 5 + 5));
     }
   printf ("\n};\n");
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
 
 #else /* !CRCTAB */
@@ -108,7 +108,6 @@ main ()
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
-#include "version.h"
 #include "error.h"
 
 /* Number of bytes to read at once.  */
@@ -250,7 +249,7 @@ cksum (char *file, int print_name)
 
   crc = ~crc & 0xFFFFFFFF;
 
-  printf ("%10lu %8ld", crc, length);
+  printf ("%lu %ld", crc, length);
   if (print_name)
     printf (" %s", file);
   putchar ('\n');
@@ -277,16 +276,20 @@ Print CRC checksum and byte counts of each FILE.\n\
   --version   output version information and exit\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
-void
+int
 main (int argc, char **argv)
 {
   int i, c;
   int errors = 0;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   have_read_stdin = 0;
 
   while ((c = getopt_long (argc, argv, "", long_options, (int *) 0)) != EOF)
@@ -303,8 +306,8 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("cksum - %s\n", version_string);
-      exit (0);
+      printf ("cksum - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
@@ -323,8 +326,8 @@ main (int argc, char **argv)
     }
 
   if (have_read_stdin && fclose (stdin) == EOF)
-    error (1, errno, "-");
-  exit (errors);
+    error (EXIT_FAILURE, errno, "-");
+  exit (errors == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 #endif /* !CRCTAB */
