@@ -1,5 +1,5 @@
 /* comm -- compare two sorted files line by line.
-   Copyright (C) 1986, 1990, 1991, 1995 Free Software Foundation, Inc.
+   Copyright (C) 1986, 1990, 1991, 1995, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 #include <sys/types.h>
 #include "system.h"
 #include "linebuffer.h"
-#include "version.h"
 #include "error.h"
 
 /* Undefine, to avoid warning about redefinition on some systems.  */
@@ -80,7 +79,7 @@ Compare sorted files LEFT_FILE and RIGHT_FILE line by line.\n\
       --version   output version information and exit\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 /* Output the line in linebuffer LINE to stream STREAM
@@ -212,12 +211,15 @@ compare_files (char **infiles)
   return ret;
 }
 
-void
+int
 main (int argc, char **argv)
 {
   int c;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
   only_file_1 = 1;
   only_file_2 = 1;
@@ -247,8 +249,8 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("comm - %s\n", version_string);
-      exit (0);
+      printf ("comm - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
@@ -257,5 +259,6 @@ main (int argc, char **argv)
   if (optind + 2 != argc)
     usage (1);
 
-  exit (compare_files (argv + optind));
+  exit (compare_files (argv + optind) == 0
+	? EXIT_SUCCESS : EXIT_FAILURE);
 }
