@@ -1,5 +1,5 @@
 /* csplit - split a file into sections determined by context lines
-   Copyright (C) 1991, 1995 Free Software Foundation, Inc.
+   Copyright (C) 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,6 +21,11 @@
 #include <config.h>
 
 #include <stdio.h>
+
+/* Disable assertions.  Some systems have broken assert macros.  */
+#define NDEBUG 1
+
+#include <assert.h>
 #include <getopt.h>
 #include <sys/types.h>
 #include <signal.h>
@@ -42,7 +47,6 @@
 # include <rx.h>
 #endif
 #include "system.h"
-#include "version.h"
 #include "error.h"
 #include "xstrtoul.h"
 
@@ -255,7 +259,7 @@ static void
 cleanup_fatal (void)
 {
   cleanup ();
-  exit (1);
+  exit (EXIT_FAILURE);
 }
 
 static RETSIGTYPE
@@ -642,7 +646,7 @@ static unsigned int
 get_first_line_in_buffer (void)
 {
   if (head == NULL && !load_buffer ())
-    error (1, errno, _("input disappeared"));
+    error (EXIT_FAILURE, errno, _("input disappeared"));
 
   return head->first_available;
 }
@@ -743,7 +747,7 @@ set_input_file (const char *name)
     {
       input_desc = open (name, O_RDONLY);
       if (input_desc < 0)
-	error (1, errno, "%s", name);
+	error (EXIT_FAILURE, errno, "%s", name);
     }
 }
 
@@ -826,10 +830,11 @@ process_line_count (const struct control *p, int repetition)
 
   linenum = get_first_line_in_buffer ();
 
-  /* Check for requesting a line that has already been written out.
-     If this ever happens, it's due to a bug in csplit. */
-  if (linenum >= last_line_to_save)
-    abort ();
+  /* Initially, I wanted to assert linenum < last_line_to_save, but that
+     condition is false for the valid command: echo | csplit - 1 '{*}'.
+     So, relax it just a little.  */
+  assert ((linenum == 1 && last_line_to_save == 1)
+	  || linenum < last_line_to_save);
 
   while (linenum++ < last_line_to_save)
     {
@@ -899,7 +904,7 @@ process_regexp (struct control *p, int repetition)
 		      dump_rest_of_file ();
 		      close_output_file ();
 		    }
-		  exit (0);
+		  exit (EXIT_SUCCESS);
 		}
 	      else
 		regexp_error (p, repetition, ignore);
@@ -939,7 +944,7 @@ process_regexp (struct control *p, int repetition)
 		      dump_rest_of_file ();
 		      close_output_file ();
 		    }
-		  exit (0);
+		  exit (EXIT_SUCCESS);
 		}
 	      else
 		regexp_error (p, repetition, ignore);
@@ -1121,11 +1126,11 @@ check_for_offset (struct control *p, const char *str, const char *num)
   unsigned long val;
 
   if (*num != '-' && *num != '+')
-    error (1, 0, _("%s: `+' or `-' expected after delimeter"), str);
+    error (EXIT_FAILURE, 0, _("%s: `+' or `-' expected after delimeter"), str);
 
   if (xstrtoul (num + 1, NULL, 10, &val, NULL) != LONGINT_OK
       || val > UINT_MAX)
-    error (1, 0, _("%s: integer expected after `%c'"), str, *num);
+    error (EXIT_FAILURE, 0, _("%s: integer expected after `%c'"), str, *num);
   p->offset = (unsigned int) val;
 
   if (*num == '-')
@@ -1145,7 +1150,7 @@ parse_repeat_count (int argnum, struct control *p, char *str)
 
   end = str + strlen (str) - 1;
   if (*end != '}')
-    error (1, 0, _("%s: `}' is required in repeat count"), str);
+    error (EXIT_FAILURE, 0, _("%s: `}' is required in repeat count"), str);
   *end = '\0';
 
   if (str+1 == end-1 && *(str+1) == '*')
@@ -1155,7 +1160,8 @@ parse_repeat_count (int argnum, struct control *p, char *str)
       if (xstrtoul (str + 1, NULL, 10, &val, NULL) != LONGINT_OK
 	  || val > UINT_MAX)
 	{
-	  error (1, 0, _("%s}: integer required between `{' and `}'"),
+	  error (EXIT_FAILURE, 0,
+		 _("%s}: integer required between `{' and `}'"),
 		 global_argv[argnum]);
 	}
       p->repeat = (unsigned int) val;
@@ -1181,7 +1187,8 @@ extract_regexp (int argnum, boolean ignore, char *str)
 
   closing_delim = strrchr (str + 1, delim);
   if (closing_delim == NULL)
-    error (1, 0, _("%s: closing delimeter `%c' missing"), str, delim);
+    error (EXIT_FAILURE, 0,
+	   _("%s: closing delimeter `%c' missing"), str, delim);
 
   len = closing_delim - str - 1;
   p = new_control_record ();
@@ -1219,6 +1226,7 @@ parse_patterns (int argc, int start, char **argv)
   int i;			/* Index into ARGV. */
   struct control *p;		/* New control record created. */
   unsigned long val;
+  static unsigned long last_val = 0;
 
   for (i = start; i < argc; i++)
     {
@@ -1230,9 +1238,25 @@ parse_patterns (int argc, int start, char **argv)
 	{
 	  p = new_control_record ();
 	  p->argnum = i;
+
 	  if (xstrtoul (argv[i], NULL, 10, &val, NULL) != LONGINT_OK
 	      || val > INT_MAX)
-	    error (1, 0, _("%s: invalid pattern"), argv[i]);
+	    error (EXIT_FAILURE, 0, _("%s: invalid pattern"), argv[i]);
+	  if (val == 0)
+	    error (EXIT_FAILURE, 0,
+		   _("%s: line number must be greater than zero"),
+		   argv[i]);
+	  if (val < last_val)
+	    error (EXIT_FAILURE, 0,
+	     _("line number `%s' is smaller than preceding line number, %lu"),
+		   argv[i], last_val);
+
+	  if (val == last_val)
+	    error (0, 0,
+	   _("warning: line number `%s' is the same as preceding line number"),
+		   argv[i]);
+	  last_val = val;
+
 	  p->lines_required = (int) val;
 	}
 
@@ -1352,14 +1376,16 @@ get_format_conv_type (char **format_ptr)
       break;
 
     case 0:
-      error (1, 0, _("missing conversion specifier in suffix"));
+      error (EXIT_FAILURE, 0, _("missing conversion specifier in suffix"));
       break;
 
     default:
       if (ISPRINT (ch))
-        error (1, 0, _("invalid conversion specifier in suffix: %c"), ch);
+        error (EXIT_FAILURE, 0,
+	       _("invalid conversion specifier in suffix: %c"), ch);
       else
-	error (1, 0, _("invalid conversion specifier in suffix: \\%.3o"), ch);
+	error (EXIT_FAILURE, 0,
+	       _("invalid conversion specifier in suffix: \\%.3o"), ch);
     }
 }
 
@@ -1390,23 +1416,29 @@ max_out (char *format)
     }
 
   if (percents == 0)
-    error (1, 0, _("missing %% conversion specification in suffix"));
+    error (EXIT_FAILURE, 0,
+	   _("missing %% conversion specification in suffix"));
   else if (percents > 1)
-    error (1, 0, _("too many %% conversion specifications in suffix"));
+    error (EXIT_FAILURE, 0,
+	   _("too many %% conversion specifications in suffix"));
 
   return out_count;
 }
 
-void
+int
 main (int argc, char **argv)
 {
   int optc;
   unsigned long val;
 #ifdef SA_INTERRUPT
   struct sigaction oldact, newact;
-#endif				/* SA_INTERRUPT */
+#endif
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   global_argv = argv;
   controls = NULL;
   control_used = 0;
@@ -1434,7 +1466,7 @@ main (int argc, char **argv)
   sigaction (SIGTERM, NULL, &oldact);
   if (oldact.sa_handler != SIG_IGN)
     sigaction (SIGTERM, &newact, NULL);
-#else
+#else /* not SA_INTERRUPT */
   if (signal (SIGHUP, SIG_IGN) != SIG_IGN)
     signal (SIGHUP, interrupt_handler);
   if (signal (SIGINT, SIG_IGN) != SIG_IGN)
@@ -1443,7 +1475,7 @@ main (int argc, char **argv)
     signal (SIGQUIT, interrupt_handler);
   if (signal (SIGTERM, SIG_IGN) != SIG_IGN)
     signal (SIGTERM, interrupt_handler);
-#endif
+#endif /* not SA_INTERRUPT */
 
   while ((optc = getopt_long (argc, argv, "f:b:kn:sqz", longopts, (int *) 0))
 	 != EOF)
@@ -1467,7 +1499,7 @@ main (int argc, char **argv)
       case 'n':
 	if (xstrtoul (optarg, NULL, 10, &val, NULL) != LONGINT_OK
 	    || val > INT_MAX)
-	  error (1, 0, _("%s: invalid number"), optarg);
+	  error (EXIT_FAILURE, 0, _("%s: invalid number"), optarg);
 	digits = (int) val;
 	break;
 
@@ -1486,8 +1518,8 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("csplit - %s\n", version_string);
-      exit (0);
+      printf ("csplit - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
@@ -1516,7 +1548,7 @@ main (int argc, char **argv)
       cleanup_fatal ();
     }
 
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
 
 static void
@@ -1555,5 +1587,5 @@ Read standard input if FILE is -.  Each PATTERN may be:\n\
 A line OFFSET is a required `+' or `-' followed by a positive integer.\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
