@@ -72,7 +72,6 @@
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
-#include "version.h"
 #include "error.h"
 
 #define FATAL_ERROR(s)							\
@@ -222,7 +221,7 @@ range, or many ranges separated by commas.  Each range is one of:\n\
 With no FILE, or when FILE is -, read standard input.\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 /* The following function was copied from getline.c, but with these changes:
@@ -670,12 +669,15 @@ cut_file (char *file)
   return 0;
 }
 
-void
+int
 main (int argc, char **argv)
 {
   int optc, exit_status = 0;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
   operating_mode = undefined_mode;
 
@@ -734,8 +736,8 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("cut - %s\n", version_string);
-      exit (0);
+      printf ("cut - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
@@ -766,7 +768,7 @@ main (int argc, char **argv)
       exit_status = 1;
     }
   if (ferror (stdout) || fclose (stdout) == EOF)
-    error (1, errno, _("write error"));
+    error (EXIT_FAILURE, errno, _("write error"));
 
-  exit (exit_status);
+  exit (exit_status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
