@@ -1,5 +1,5 @@
 /* expand - convert tabs to spaces
-   Copyright (C) 1989, 1991, 1995 Free Software Foundation, Inc.
+   Copyright (C) 89, 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -42,7 +42,6 @@
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
-#include "version.h"
 #include "error.h"
 
 /* The number of bytes added at a time to the amount of memory
@@ -130,7 +129,7 @@ With no FILE, or when FILE is -, read standard input.\n\
 Instead of -t NUMBER or -t LIST, -NUMBER or -LIST may be used.\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 /* Add tab stop TABVAL to the end of `tab_list', except
@@ -169,7 +168,7 @@ parse_tabstops (char *stops)
 	  tabval = tabval * 10 + *stops - '0';
 	}
       else
-	error (1, 0, _("tab size contains an invalid character"));
+	error (EXIT_FAILURE, 0, _("tab size contains an invalid character"));
     }
 
   add_tabstop (tabval);
@@ -187,9 +186,9 @@ validate_tabstops (int *tabs, int entries)
   for (i = 0; i < entries; i++)
     {
       if (tabs[i] == 0)
-	error (1, 0, _("tab size cannot be 0"));
+	error (EXIT_FAILURE, 0, _("tab size cannot be 0"));
       if (tabs[i] <= prev_tab)
-	error (1, 0, _("tab sizes must be ascending"));
+	error (EXIT_FAILURE, 0, _("tab sizes must be ascending"));
       prev_tab = tabs[i];
     }
 }
@@ -322,7 +321,7 @@ expand (void)
     }
 }
 
-void
+int
 main (int argc, char **argv)
 {
   int tabval = -1;		/* Value of tabstop being read, or -1. */
@@ -334,6 +333,9 @@ main (int argc, char **argv)
   tab_list = NULL;
   first_free_tab = 0;
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
   while ((c = getopt_long (argc, argv, "it:,0123456789", longopts, (int *) 0))
 	 != EOF)
@@ -365,8 +367,8 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("expand - %s\n", version_string);
-      exit (0);
+      printf ("expand - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
@@ -391,9 +393,9 @@ main (int argc, char **argv)
   expand ();
 
   if (have_read_stdin && fclose (stdin) == EOF)
-    error (1, errno, "-");
+    error (EXIT_FAILURE, errno, "-");
   if (ferror (stdout) || fclose (stdout) == EOF)
-    error (1, errno, _("write error"));
+    error (EXIT_FAILURE, errno, _("write error"));
 
-  exit (exit_status);
+  exit (exit_status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
