@@ -1,5 +1,5 @@
 /* fold -- wrap each input line to fit in specified width.
-   Copyright (C) 1991, 1995 Free Software Foundation, Inc.
+   Copyright (C) 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -39,7 +39,6 @@
 #endif
 
 #include "system.h"
-#include "version.h"
 #include "xstrtol.h"
 #include "error.h"
 
@@ -95,7 +94,7 @@ standard output.\n\
   -w, --width=WIDTH   use WIDTH columns instead of 80\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 /* Assuming the current column is COLUMN, return the column that
@@ -246,7 +245,7 @@ fold_file (char *filename, int width)
   return 0;
 }
 
-void
+int
 main (int argc, char **argv)
 {
   int width = 80;
@@ -255,6 +254,10 @@ main (int argc, char **argv)
   int errs = 0;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   break_spaces = count_bytes = have_read_stdin = 0;
 
   /* Turn any numeric options into -w options.  */
@@ -293,7 +296,8 @@ main (int argc, char **argv)
 	    long int tmp_long;
 	    if (xstrtol (optarg, NULL, 10, &tmp_long, NULL) != LONGINT_OK
 		|| tmp_long <= 0 || tmp_long > INT_MAX)
-	      error (1, 0, _("invalid number of columns: `%s'"), optarg);
+	      error (EXIT_FAILURE, 0,
+		     _("invalid number of columns: `%s'"), optarg);
 	    width = (int) tmp_long;
 	  }
 	  break;
@@ -305,8 +309,8 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("fold - %s\n", version_string);
-      exit (0);
+      printf ("fold - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
@@ -319,9 +323,9 @@ main (int argc, char **argv)
       errs |= fold_file (argv[i], width);
 
   if (have_read_stdin && fclose (stdin) == EOF)
-    error (1, errno, "-");
+    error (EXIT_FAILURE, errno, "-");
   if (fclose (stdout) == EOF)
-    error (1, errno, _("write error"));
+    error (EXIT_FAILURE, errno, _("write error"));
 
-  exit (errs);
+  exit (errs == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
