@@ -1,5 +1,5 @@
 /* head -- output first part of file(s)
-   Copyright (C) 1989, 1990, 1991, 1995 Free Software Foundation, Inc.
+   Copyright (C) 89, 90, 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,7 +30,6 @@
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
-#include "version.h"
 #include "error.h"
 
 /* Number of lines/chars/blocks to head. */
@@ -107,7 +106,7 @@ If -VALUE is used as first OPTION, read -c VALUE when one of\n\
 multipliers bkm follows concatenated, else read -n VALUE.\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 /* Convert STR, a string of ASCII digits, into an unsigned integer.
@@ -176,7 +175,7 @@ head_bytes (const char *filename, int fd, long int bytes_to_write)
       if (bytes_read > bytes_to_write)
 	bytes_read = bytes_to_write;
       if (fwrite (buffer, 1, bytes_read, stdout) == 0)
-	error (1, errno, _("write error"));
+	error (EXIT_FAILURE, errno, _("write error"));
       bytes_to_write -= bytes_read;
     }
   return 0;
@@ -204,7 +203,7 @@ head_lines (const char *filename, int fd, long int lines_to_write)
 	if (buffer[bytes_to_write++] == '\n' && --lines_to_write == 0)
 	  break;
       if (fwrite (buffer, 1, bytes_to_write, stdout) == 0)
-	error (1, errno, _("write error"));
+	error (EXIT_FAILURE, errno, _("write error"));
     }
   return 0;
 }
@@ -249,7 +248,7 @@ head_file (const char *filename, long int number)
     }
 }
 
-void
+int
 main (int argc, char **argv)
 {
   enum header_mode header_mode = multiple_files;
@@ -258,6 +257,10 @@ main (int argc, char **argv)
   int c;			/* Option character. */
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   have_read_stdin = 0;
   unit_size = 0;
   print_headers = 0;
@@ -331,7 +334,7 @@ main (int argc, char **argv)
 	  /* FIXME: use xstrtoul instead.  */
 	  number = atou (optarg);
 	  if (number == -1)
-	    error (1, 0, _("invalid number `%s'"), optarg);
+	    error (EXIT_FAILURE, 0, _("invalid number `%s'"), optarg);
 	  break;
 
 	case 'q':
@@ -349,8 +352,8 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("head - %s\n", version_string);
-      exit (0);
+      printf ("head - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
@@ -373,9 +376,9 @@ main (int argc, char **argv)
     exit_status |= head_file (argv[optind], number);
 
   if (have_read_stdin && close (0) < 0)
-    error (1, errno, "-");
+    error (EXIT_FAILURE, errno, "-");
   if (fclose (stdout) == EOF)
-    error (1, errno, _("write error"));
+    error (EXIT_FAILURE, errno, _("write error"));
 
-  exit (exit_status);
+  exit (exit_status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
