@@ -1,5 +1,5 @@
 /* join - join lines of two files on a common field
-   Copyright (C) 1991, 1995 Free Software Foundation, Inc.
+   Copyright (C) 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -54,11 +54,17 @@ char *alloca ();
 # define INT_MAX ((int) (UINT_MAX >> 1))
 #endif
 
+#if _LIBC || STDC_HEADERS
+# define TOLOWER(c) tolower (c)
+#else
+# define TOLOWER(c) (ISUPPER (c) ? tolower (c) : (c))
+#endif
+
 #include "system.h"
-#include "version.h"
 #include "long-options.h"
 #include "xstrtol.h"
 #include "error.h"
+#include "memcasecmp.h"
 
 #define join system_join
 
@@ -141,6 +147,7 @@ static char tab;
    a character that is a short option.  */
 static struct option const longopts[] =
 {
+  {"ignore-case", no_argument, NULL, 'i'},
   {"j", required_argument, NULL, 'j'},
   {"j1", required_argument, NULL, '1'},
   {"j2", required_argument, NULL, '2'},
@@ -150,6 +157,9 @@ static struct option const longopts[] =
 /* Used to print non-joining lines */
 static struct line uni_blank;
 
+/* If nonzero, ignore case when comparing join fields.  */
+static int ignore_case;
+
 static void
 usage (int status)
 {
@@ -169,6 +179,7 @@ by whitespace.  When FILE1 or FILE2 (not both) is -, read standard input.\n\
 \n\
   -a SIDE          print unpairable lines coming from file SIDE\n\
   -e EMPTY         replace missing input fields with EMPTY\n\
+  -i, --ignore-case ignore differences in case when comparing fields\n\
   -j FIELD         (Obsolescent) equivalent to `-1 FIELD -2 FIELD'\n\
   -j1 FIELD        (Obsolescent) equivalent to `-1 FIELD'\n\
   -j2 FIELD        (Obsolescent) equivalent to `-2 FIELD'\n\
@@ -188,7 +199,7 @@ the remaining fields from FILE1, the remaining fields from FILE2, all\n\
 separated by CHAR.\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 static void
@@ -217,6 +228,13 @@ xfields (struct line *line)
   ptr = line->beg;
   lim = line->lim;
 
+  if (!tab)
+    {
+      /* Skip leading blanks before the first field.  */
+      while (ptr < lim && ISSPACE (*ptr))
+	++ptr;
+    }
+
   for (i = 0; ptr < lim; ++i)
     {
       if (tab)
@@ -373,7 +391,15 @@ keycmp (struct line *line1, struct line *line2)
     return len2 == 0 ? 0 : -1;
   if (len2 == 0)
     return 1;
-  diff = memcmp (beg1, beg2, min (len1, len2));
+
+  /* Use an if-statement here rather than a function variable to
+     avoid portability hassles of getting a non-conflicting declaration
+     of memcmp.  */
+  if (ignore_case)
+    diff = memcasecmp (beg1, beg2, min (len1, len2));
+  else
+    diff = memcmp (beg1, beg2, min (len1, len2));
+
   if (diff)
     return diff;
   return len1 - len2;
@@ -623,7 +649,7 @@ add_field (int file, int field)
 static int
 decode_field_spec (const char *s, int *file_index, int *field_index)
 {
-  int valid = 0;
+  int invalid = 1;
 
   /* The first character must be 0, 1, or 2.  */
   switch (s[0])
@@ -633,7 +659,7 @@ decode_field_spec (const char *s, int *file_index, int *field_index)
 	{
 	  *file_index = 0;
 	  /* Leave *field_index undefined.  */
-	  valid = 1;
+	  invalid = 0;
 	}
       else
         {
@@ -659,7 +685,7 @@ decode_field_spec (const char *s, int *file_index, int *field_index)
 	      *file_index = s[0] - '0';
 	      /* Convert to a zero-based index.  */
 	      *field_index = (int) tmp_long - 1;
-	      valid = 1;
+	      invalid = 0;
 	    }
 	}
       break;
@@ -668,7 +694,7 @@ decode_field_spec (const char *s, int *file_index, int *field_index)
       error (0, 0, _("invalid file number in field spec: `%s'"), s);
       break;
     }
-  return !valid;
+  return invalid;
 }
 
 /* Add the comma or blank separated field spec(s) in STR to `outlist'.
@@ -722,7 +748,7 @@ make_blank (struct line *blank, int count)
   blank->lim = &blank->beg[i];
 }
 
-void
+int
 main (int argc, char **argv)
 {
   char *names[2];
@@ -730,17 +756,20 @@ main (int argc, char **argv)
   int optc, prev_optc = 0, nfiles;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
   /* Initialize this before parsing options.  In parsing options,
      it may be increased.  */
   uni_blank.nfields = 1;
 
-  parse_long_options (argc, argv, "join", version_string, usage);
+  parse_long_options (argc, argv, "join", PACKAGE_VERSION, usage);
 
   nfiles = 0;
   print_pairables = 1;
 
-  while ((optc = getopt_long_only (argc, argv, "-a:e:1:2:o:t:v:", longopts,
+  while ((optc = getopt_long_only (argc, argv, "-a:e:i1:2:o:t:v:", longopts,
 				   (int *) 0)) != EOF)
     {
       long int val;
@@ -757,7 +786,7 @@ main (int argc, char **argv)
 	case 'a':
 	  if (xstrtol (optarg, NULL, 10, &val, NULL) != LONGINT_OK
 	      || (val != 1 && val != 2))
-	    error (2, 0, _("invalid field number: `%s'"), optarg);
+	    error (EXIT_FAILURE, 0, _("invalid field number: `%s'"), optarg);
 	  if (val == 1)
 	    print_unpairables_1 = 1;
 	  else
@@ -768,11 +797,16 @@ main (int argc, char **argv)
 	  empty_filler = optarg;
 	  break;
 
+	case 'i':
+	  ignore_case = 1;
+	  break;
+
 	case '1':
 	  if (xstrtol (optarg, NULL, 10, &val, NULL) != LONGINT_OK
 	      || val <= 0 || val > INT_MAX)
 	    {
-	      error (2, 0, _("invalid field number for file 1: `%s'"), optarg);
+	      error (EXIT_FAILURE, 0,
+		     _("invalid field number for file 1: `%s'"), optarg);
 	    }
 	  join_field_1 = (int) val - 1;
 	  break;
@@ -780,20 +814,21 @@ main (int argc, char **argv)
 	case '2':
 	  if (xstrtol (optarg, NULL, 10, &val, NULL) != LONGINT_OK
 	      || val <= 0 || val > INT_MAX)
-	    error (2, 0, _("invalid field number for file 2: `%s'"), optarg);
+	    error (EXIT_FAILURE, 0,
+		   _("invalid field number for file 2: `%s'"), optarg);
 	  join_field_2 = (int) val - 1;
 	  break;
 
 	case 'j':
 	  if (xstrtol (optarg, NULL, 10, &val, NULL) != LONGINT_OK
 	      || val <= 0 || val > INT_MAX)
-	    error (2, 0, _("invalid field number: `%s'"), optarg);
+	    error (EXIT_FAILURE, 0, _("invalid field number: `%s'"), optarg);
 	  join_field_1 = join_field_2 = (int) val - 1;
 	  break;
 
 	case 'o':
 	  if (add_field_list (optarg))
-	    exit (1);
+	    exit (EXIT_FAILURE);
 	  break;
 
 	case 't':
@@ -804,7 +839,7 @@ main (int argc, char **argv)
 	  if (prev_optc == 'o' && optind <= argc - 2)
 	    {
 	      if (add_field_list (optarg))
-		exit (1);
+		exit (EXIT_FAILURE);
 
 	      /* Might be continuation of args to -o.  */
 	      continue;		/* Don't change `prev_optc'.  */
@@ -812,7 +847,7 @@ main (int argc, char **argv)
 
 	  if (nfiles > 1)
 	    {
-	      error (0, 0, "too many non-option arguments");
+	      error (0, 0, _("too many non-option arguments"));
 	      usage (1);
 	    }
 	  names[nfiles++] = optarg;
@@ -830,28 +865,28 @@ main (int argc, char **argv)
 
   if (nfiles != 2)
     {
-      error (0, 0, "too few non-option arguments");
+      error (0, 0, _("too few non-option arguments"));
       usage (1);
     }
 
   fp1 = strcmp (names[0], "-") ? fopen (names[0], "r") : stdin;
   if (!fp1)
-    error (1, errno, "%s", names[0]);
+    error (EXIT_FAILURE, errno, "%s", names[0]);
   fp2 = strcmp (names[1], "-") ? fopen (names[1], "r") : stdin;
   if (!fp2)
-    error (1, errno, "%s", names[1]);
+    error (EXIT_FAILURE, errno, "%s", names[1]);
   if (fp1 == fp2)
-    error (1, errno, _("both files cannot be standard input"));
+    error (EXIT_FAILURE, errno, _("both files cannot be standard input"));
   join (fp1, fp2);
 
   if (fp1 != stdin && fclose (fp1) == EOF)
-    error (1, errno, "%s", names[0]);
+    error (EXIT_FAILURE, errno, "%s", names[0]);
   if (fp2 != stdin && fclose (fp2) == EOF)
-    error (1, errno, "%s", names[1]);
+    error (EXIT_FAILURE, errno, "%s", names[1]);
   if ((fp1 == stdin || fp2 == stdin) && fclose (stdin) == EOF)
-    error (1, errno, "-");
+    error (EXIT_FAILURE, errno, "-");
   if (ferror (stdout) || fclose (stdout) == EOF)
-    error (1, errno, _("write error"));
+    error (EXIT_FAILURE, errno, _("write error"));
 
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
