@@ -1,6 +1,6 @@
 /* Compute MD5 checksum of files or strings according to the definition
    of MD5 in RFC 1321 from April 1992.
-   Copyright (C) 1995 Free Software Foundation, Inc.
+   Copyright (C) 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,7 +31,6 @@
 #include "getline.h"
 #include "system.h"
 #include "error.h"
-#include "version.h"
 
 /* Most systems do not distinguish between external and internal
    text representations.  */
@@ -60,6 +59,13 @@
 # define TOLOWER(c) (ISUPPER (c) ? tolower (c) : (c))
 #endif
 
+/* The minimum length of a valid digest line in a file produced
+   by `md5sum FILE' and read by `md5sum --check'.  This length does
+   not include any newline character at the end of a line.  */
+#define MIN_DIGEST_LINE_LENGTH (32 /* message digest length */ \
+				+ 2 /* blank and binary indicator */ \
+				+ 1 /* minimum filename length */ )
+
 /* Nonzero if any of the files read were the standard input. */
 static int have_read_stdin;
 
@@ -119,15 +125,14 @@ a line with checksum, a character indicating type (`*' for binary, ` ' for\n\
 text), and name for each FILE.\n"),
 	    program_name, program_name, program_name);
 
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
-/* FIXME: this format loses with filenames containing newline.  */
-
 static int
-split_3 (char *s, char **u, int *binary, char **w)
+split_3 (char *s, size_t s_len, char **u, int *binary, char **w)
 {
   size_t i;
+  int filename_has_newline = 0;
 
 #define ISWHITE(c) ((c) == ' ' || (c) == '\t')
 
@@ -135,34 +140,82 @@ split_3 (char *s, char **u, int *binary, char **w)
   while (ISWHITE (s[i]))
     ++i;
 
-  /* The line has to be at least 35 characters long to contain correct
-     message digest information.  */
-  if (strlen (&s[i]) >= 35)
+  /* The line must have at least 35 (36 if the first is a backslash)
+     more characters to contain correct message digest information.
+     Ignore this line if it is too short.  */
+  if (!(s_len - i >= MIN_DIGEST_LINE_LENGTH
+	|| (s[i] == '\\' && s_len - i >= 1 + MIN_DIGEST_LINE_LENGTH)))
+    return 1;
+
+  if (s[i] == '\\')
     {
-      *u = &s[i];
-
-      /* The first field has to be the 32-character hexadecimal
-	 representation of the message digest.  If it not immediately
-	 followed by a white space it's an error.  */
-      i += 32;
-      if (!ISWHITE (s[i]))
-	return 1;
-
-      s[i++] = '\0';
-
-      if (s[i] != ' ' && s[i] != '*')
-	return 1;
-      *binary = s[i++] == '*';
-
-      /* All characters between the type indicator and end of line are
-	 significant -- that includes leading and trailing white space.  */
-      *w = &s[i];
-
-      /* So this line is valid as long as there is at least one character
-	 for the filename.  */
-      return (**w ? 0 : 1);
+      ++i;
+      filename_has_newline = 1;
     }
-  return 1;
+  *u = &s[i];
+
+  /* The first field has to be the 32-character hexadecimal
+     representation of the message digest.  If it is not followed
+     immediately by a white space it's an error.  */
+  i += 32;
+  if (!ISWHITE (s[i]))
+    return 1;
+
+  s[i++] = '\0';
+
+  if (s[i] != ' ' && s[i] != '*')
+    return 1;
+  *binary = (s[i++] == '*');
+
+  /* All characters between the type indicator and end of line are
+     significant -- that includes leading and trailing white space.  */
+  *w = &s[i];
+
+  if (filename_has_newline)
+    {
+      /* Translate each `\n' string in the file name to a NEWLINE,
+	 and each `\\' string to a backslash.  */
+
+      char *dst = &s[i];
+
+      while (i < s_len)
+	{
+	  switch (s[i])
+	    {
+	    case '\\':
+	      if (i == s_len - 1)
+		{
+		  /* A valid line does not end with a backslash.  */
+		  return 1;
+		}
+	      ++i;
+	      switch (s[i++])
+		{
+		case 'n':
+		  *dst++ = '\n';
+		  break;
+		case '\\':
+		  *dst++ = '\\';
+		  break;
+		default:
+		  /* Only `\' or `n' may follow a backslash.  */
+		  return 1;
+		}
+	      break;
+
+	    case '\0':
+	      /* The file name may not contain a NUL.  */
+	      return 1;
+	      break;
+
+	    default:
+	      *dst++ = s[i++];
+	      break;
+	    }
+	}
+      *dst = '\0';
+    }
+  return 0;
 }
 
 static int
@@ -177,7 +230,9 @@ hex_digits (const char *s)
   return 1;
 }
 
-/* FIXME: allow newline in filename by encoding it. */
+/* An interface to md5_stream.  Operate on FILENAME (it may be "-") and
+   put the result in *MD5_RESULT.  Return non-zero upon failure, zero
+   to indicate success.  */
 
 static int
 md5_file (const char *filename, int binary, unsigned char *md5_result)
@@ -275,7 +330,7 @@ md5_check (const char *checkfile_name, int binary)
       if (line[line_length - 1] == '\n')
 	line[--line_length] = '\0';
 
-      err = split_3 (line, &md5num, &type_flag, &filename);
+      err = split_3 (line, line_length, &md5num, &type_flag, &filename);
       if (err || !hex_digits (md5num))
 	{
 	  if (warn)
@@ -337,7 +392,7 @@ md5_check (const char *checkfile_name, int binary)
 
   if (ferror (checkfile_stream))
     {
-      error (0, 0, "%s: read error", checkfile_name);
+      error (0, 0, _("%s: read error"), checkfile_name);
       return 1;
     }
 
@@ -363,17 +418,19 @@ md5_check (const char *checkfile_name, int binary)
 	  if (n_open_or_read_failures > 0)
 	    {
 	      error (0, 0,
-		   _("WARNING: %d of %d listed file%s could not be read\n"),
+		   _("WARNING: %d of %d listed %s could not be read\n"),
 		     n_open_or_read_failures, n_properly_formated_lines,
-		     (n_properly_formated_lines == 1 ? "" : "s"));
+		     (n_properly_formated_lines == 1
+		      ? _("file") : _("files")));
 	    }
 
 	  if (n_mismatched_checksums > 0)
 	    {
 	      error (0, 0,
-		   _("WARNING: %d of %d computed checksum%s did NOT match\n"),
+		   _("WARNING: %d of %d computed %s did NOT match"),
 		     n_mismatched_checksums, n_computed_checkums,
-		     (n_computed_checkums == 1 ? "" : "s"));
+		     (n_computed_checkums == 1
+		      ? _("checksum") : _("checksums")));
 	    }
 	}
     }
@@ -399,8 +456,11 @@ main (int argc, char **argv)
 
   /* Setting values of global variables.  */
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
-  parse_long_options (argc, argv, "md5sum", version_string, usage);
+  parse_long_options (argc, argv, "md5sum", PACKAGE_VERSION, usage);
 
   while ((opt = getopt_long (argc, argv, "bctw", long_options, NULL))
 	 != EOF)
@@ -441,7 +501,7 @@ main (int argc, char **argv)
 
   if (do_version)
     {
-      printf ("md5sum - %s\n", version_string);
+      printf ("md5sum - %s\n", PACKAGE_VERSION);
       exit (EXIT_SUCCESS);
     }
 
@@ -502,17 +562,49 @@ main (int argc, char **argv)
 
       for (; optind < argc; ++optind)
 	{
-	  size_t i;
 	  int fail;
+	  char *file = argv[optind];
 
-	  fail = md5_file (argv[optind], binary, md5buffer);
+	  fail = md5_file (file, binary, md5buffer);
 	  err |= fail;
 	  if (!fail)
 	    {
+	      size_t i;
+
+	      /* Output a leading backslash if the file name contains
+		 a newline.  */
+	      if (strchr (file, '\n'))
+		putchar ('\\');
+
 	      for (i = 0; i < 16; ++i)
 		printf ("%02x", md5buffer[i]);
 
-	      printf (" %c%s\n", binary ? '*' : ' ', argv[optind]);
+	      putchar (' ');
+	      if (binary)
+		putchar ('*');
+	      else
+		putchar (' ');
+
+	      /* Translate each NEWLINE byte to the string, "\\n",
+		 and each backslash to "\\\\".  */
+	      for (i = 0; i < strlen (file); ++i)
+		{
+		  switch (file[i])
+		    {
+		    case '\n':
+		      fputs ("\\n", stdout);
+		      break;
+
+		    case '\\':
+		      fputs ("\\\\", stdout);
+		      break;
+
+		    default:
+		      putchar (file[i]);
+		      break;
+		    }
+		}
+	      putchar ('\n');
 	    }
 	}
     }
