@@ -1,5 +1,5 @@
 /* nl -- number lines of files
-   Copyright (C) 1989, 1992, 1995 Free Software Foundation, Inc.
+   Copyright (C) 89, 92, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,7 +44,6 @@
 
 #include "linebuffer.h"
 #include "system.h"
-#include "version.h"
 #include "error.h"
 #include "xstrtol.h"
 
@@ -231,7 +230,7 @@ FORMAT is one of:\n\
 \n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 /* Build the printf format string, based on `lineno_format'. */
@@ -282,7 +281,7 @@ build_type_arg (char **typep, struct re_pattern_buffer *regexp)
       regexp->fastmap_accurate = 0;
       errmsg = re_compile_pattern (optarg, optlen, regexp);
       if (errmsg)
-	error (1, 0, "%s", errmsg);
+	error (EXIT_FAILURE, 0, "%s", errmsg);
       break;
     default:
       rval = FALSE;
@@ -460,12 +459,16 @@ nl_file (const char *file)
   return 0;
 }
 
-void
+int
 main (int argc, char **argv)
 {
   int c, exit_status = 0;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   have_read_stdin = 0;
 
   while ((c = getopt_long (argc, argv, "h:b:f:v:i:pl:s:w:n:d:", longopts,
@@ -494,7 +497,7 @@ main (int argc, char **argv)
 	    if (xstrtol (optarg, NULL, 10, &tmp_long, NULL) != LONGINT_OK
 		/* Allow it to be negative.  */
 		|| tmp_long > INT_MAX)
-	      error (1, 0, _("invalid starting line number: `%s'"),
+	      error (EXIT_FAILURE, 0, _("invalid starting line number: `%s'"),
 		     optarg);
 	    starting_line_number = (int) tmp_long;
 	  }
@@ -504,7 +507,7 @@ main (int argc, char **argv)
 	    long int tmp_long;
 	    if (xstrtol (optarg, NULL, 10, &tmp_long, NULL) != LONGINT_OK
 		|| tmp_long <= 0 || tmp_long > INT_MAX)
-	      error (1, 0, _("invalid line number increment: `%s'"),
+	      error (EXIT_FAILURE, 0, _("invalid line number increment: `%s'"),
 		     optarg);
 	    page_incr = (int) tmp_long;
 	  }
@@ -517,7 +520,7 @@ main (int argc, char **argv)
 	    long int tmp_long;
 	    if (xstrtol (optarg, NULL, 10, &tmp_long, NULL) != LONGINT_OK
 		|| tmp_long <= 0 || tmp_long > INT_MAX)
-	      error (1, 0, _("invalid number of blank lines: `%s'"),
+	      error (EXIT_FAILURE, 0, _("invalid number of blank lines: `%s'"),
 		     optarg);
 	    blank_join = (int) tmp_long;
 	  }
@@ -530,7 +533,8 @@ main (int argc, char **argv)
 	    long int tmp_long;
 	    if (xstrtol (optarg, NULL, 10, &tmp_long, NULL) != LONGINT_OK
 		|| tmp_long <= 0 || tmp_long > INT_MAX)
-	      error (1, 0, _("invalid line number field width: `%s'"),
+	      error (EXIT_FAILURE, 0,
+		     _("invalid line number field width: `%s'"),
 		     optarg);
 	    lineno_width = (int) tmp_long;
 	  }
@@ -574,8 +578,8 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("nl - %s\n", version_string);
-      exit (0);
+      printf ("nl - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
@@ -624,7 +628,7 @@ main (int argc, char **argv)
       exit_status = 1;
     }
   if (ferror (stdout) || fclose (stdout) == EOF)
-    error (1, errno, _("write error"));
+    error (EXIT_FAILURE, errno, _("write error"));
 
-  exit (exit_status);
+  exit (exit_status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
