@@ -1,5 +1,5 @@
 /* od -- dump files in octal and other formats
-   Copyright (C) 1992, 1995 Free Software Foundation, Inc.
+   Copyright (C) 92, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,7 +29,12 @@
 #ifdef _AIX
  #pragma alloca
 #else /* not _AIX */
+#ifdef _WIN32
+#include <malloc.h>
+#include <io.h>
+#else
 char *alloca ();
+#endif /* not _WIN32 */
 #endif /* not _AIX */
 #endif /* not HAVE_ALLOCA_H */
 #endif /* not __GNUC__ */
@@ -39,7 +44,6 @@ char *alloca ();
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
-#include "version.h"
 #include "xstrtoul.h"
 #include "error.h"
 
@@ -71,10 +75,17 @@ typedef double LONG_DOUBLE;
 #ifndef ULONG_MAX
 # define ULONG_MAX ((unsigned long) ~(unsigned long) 0)
 #endif
+
+#if HAVE_VALUES_H
+# include <values.h>
+#endif
+
+#ifndef BITSPERBYTE
+# define BITSPERBYTE 8
+#endif
+
 #ifndef OFF_T_MAX
-/* FIXME: is there a way to do this without relying on the
-   `8 bits per byte' assumption?  */
-# define OFF_T_MAX (~((off_t)1 << (sizeof (off_t) * 8 - 1)))
+# define OFF_T_MAX ((off_t)(~((off_t)1 << (sizeof (off_t) * BITSPERBYTE - 1))))
 #endif
 
 #define STREQ(a,b) (strcmp((a), (b)) == 0)
@@ -87,6 +98,22 @@ typedef double LONG_DOUBLE;
 # define MIN(a,b) (((a) < (b)) ? (a) : (b))
 #endif
 
+#if defined (_WIN32) && defined (_O_BINARY)
+# define WINDOWS_SETFILEMODE_BINARY(IN_STREAM, FILENAME)		\
+    do									\
+      { /* Turn off DOS text file modes, "rb" doesn't work on stdin.  */\
+	if (_setmode (_fileno ((IN_STREAM)), _O_BINARY) == -1)		\
+          {								\
+	    error (0, errno, "%s", (FILENAME));				\
+	    err = 1;							\
+	    continue;							\
+	  }								\
+      }									\
+    while (0)
+#else
+# define WINDOWS_SETFILEMODE_BINARY(IN_STREAM, FILENAME) /* empty */
+#endif
+
 /* The default number of input bytes per output line.  */
 #define DEFAULT_BYTES_PER_BLOCK 16
 
@@ -366,7 +393,7 @@ number implies 3.  -w without a number implies 32.  By default, od\n\
 uses -A o -t d2 -w 16.\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 /* Compute the greatest common denominator of U and V
@@ -629,22 +656,25 @@ simple_strtoul (const char *s, const char **p, long unsigned int *val)
   return 0;
 }
 
-/* If S points to a single valid POSIX-style od format string, put a
-   description of that format in *TSPEC, make *NEXT point at the character
-   following the just-decoded format (if *NEXT is non-NULL), and return
-   zero.  If S is not valid, don't modify *NEXT or *TSPEC and return
-   nonzero.  For example, if S were "d4afL" *NEXT would be set to "afL"
-   and *TSPEC would be
+/* If S points to a single valid POSIX-style od format string, put
+   a description of that format in *TSPEC, make *NEXT point at the
+   character following the just-decoded format (if *NEXT is non-NULL),
+   and return zero.  If S is not valid, don't modify *NEXT or *TSPEC,
+   give a diagnostic, and return nonzero.  For example, if S were
+   "d4afL" *NEXT would be set to "afL" and *TSPEC would be
      {
        fmt = SIGNED_DECIMAL;
        size = INT or LONG; (whichever integral_type_size[4] resolves to)
        print_function = print_int; (assuming size == INT)
        fmt_string = "%011d%c";
       }
+   S_ORIG is solely for reporting errors.  It should be the full format
+   string argument.
    */
 
 static int
-decode_one_format (const char *s, const char **next, struct tspec *tspec)
+decode_one_format (const char *s_orig, const char *s, const char **next,
+		   struct tspec *tspec)
 {
   enum size_spec size_spec;
   unsigned long int size;
@@ -689,14 +719,24 @@ decode_one_format (const char *s, const char **next, struct tspec *tspec)
 
 	default:
 	  if (simple_strtoul (s, &p, &size) != 0)
-	    return 1;
+	    {
+	      /* The integer at P in S would overflow an unsigned long.
+		 A digit string that long is sufficiently odd looking
+		 that the following diagnostic is sufficient.  */
+	      error (0, 0, _("invalid type string `%s'"), s_orig);
+	      return 1;
+	    }
 	  if (p == s)
 	    size = sizeof (int);
 	  else
 	    {
 	      if (size > MAX_INTEGRAL_TYPE_SIZE
 		  || integral_type_size[size] == NO_SIZE)
-		return 1;
+		{
+		  error (0, 0, _("invalid type string `%s';\n\
+this system doesn't provide a %lu-byte integral type"), s_orig, size);
+		  return 1;
+		}
 	      s = p;
 	    }
 	  break;
@@ -792,14 +832,24 @@ decode_one_format (const char *s, const char **next, struct tspec *tspec)
 
 	default:
 	  if (simple_strtoul (s, &p, &size) != 0)
-	    return 1;
+	    {
+	      /* The integer at P in S would overflow an unsigned long.
+		 A digit string that long is sufficiently odd looking
+		 that the following diagnostic is sufficient.  */
+	      error (0, 0, _("invalid type string `%s'"), s_orig);
+	      return 1;
+	    }
 	  if (p == s)
 	    size = sizeof (double);
 	  else
 	    {
 	      if (size > MAX_FP_TYPE_SIZE
 		  || fp_type_size[size] == NO_SIZE)
-		return 1;
+		{
+		  error (0, 0, _("invalid type string `%s';\n\
+this system doesn't provide a %lu-byte floating point type"), s_orig, size);
+		  return 1;
+		}
 	      s = p;
 	    }
 	  break;
@@ -828,7 +878,7 @@ decode_one_format (const char *s, const char **next, struct tspec *tspec)
 #ifdef HAVE_LONG_DOUBLE
 	case FLOAT_LONG_DOUBLE:
 	  print_function = print_long_double;
-	  pre_fmt_string = "%%%d.%dle%%c";
+	  pre_fmt_string = "%%%d.%dLe%%c";
 	  fmt_string = xmalloc (strlen (pre_fmt_string));
 	  sprintf (fmt_string, pre_fmt_string,
 		   LDBL_DIG + 8, LDBL_DIG);
@@ -857,6 +907,8 @@ decode_one_format (const char *s, const char **next, struct tspec *tspec)
       break;
 
     default:
+      error (0, 0, _("invalid character `%c' in type string `%s'"),
+	     *s, s_orig);
       return 1;
     }
 
@@ -878,6 +930,7 @@ decode_one_format (const char *s, const char **next, struct tspec *tspec)
 static int
 decode_format_string (const char *s)
 {
+  const char *s_orig = s;
   assert (s != NULL);
 
   while (*s != '\0')
@@ -885,7 +938,7 @@ decode_format_string (const char *s)
       struct tspec tspec;
       const char *next;
 
-      if (decode_one_format (s, &next, &tspec))
+      if (decode_one_format (s_orig, s, &next, &tspec))
 	return 1;
 
       assert (s != next);
@@ -898,7 +951,8 @@ decode_format_string (const char *s)
 						   * sizeof (struct tspec)));
 	}
 
-      memcpy ((char *) &spec[n_specs], (char *) &tspec, sizeof (struct tspec));
+      memcpy ((char *) &spec[n_specs], (char *) &tspec,
+	      sizeof (struct tspec));
       ++n_specs;
     }
 
@@ -941,6 +995,7 @@ skip (off_t n_skip)
 	      continue;
 	    }
 	}
+      WINDOWS_SETFILEMODE_BINARY (in_stream, input_filename);
 
       if (n_skip == 0)
 	break;
@@ -1016,7 +1071,7 @@ skip (off_t n_skip)
     }
 
   if (n_skip != 0)
-    error (2, 0, _("cannot skip past end of combined input"));
+    error (EXIT_FAILURE, 0, _("cannot skip past end of combined input"));
 
   return err;
 }
@@ -1189,6 +1244,7 @@ read_char (int *c)
 		  err = 1;
 		}
 	    }
+	  WINDOWS_SETFILEMODE_BINARY (in_stream, input_filename);
 	}
       while (in_stream == NULL);
     }
@@ -1261,6 +1317,7 @@ read_block (size_t n, char *block, size_t *n_bytes_in_buffer)
 		  err = 1;
 		}
 	    }
+	  WINDOWS_SETFILEMODE_BINARY (in_stream, input_filename);
 	}
       while (in_stream == NULL);
     }
@@ -1551,6 +1608,7 @@ main (int argc, char **argv)
   unsigned int address_pad_len;
   unsigned long int desired_width;
   int width_specified = 0;
+  int n_failed_decodes = 0;
   int err;
 
   /* The old-style `pseudo starting address' to be printed in parentheses
@@ -1562,6 +1620,10 @@ main (int argc, char **argv)
 #endif
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   err = 0;
 
   for (i = 0; i <= MAX_INTEGRAL_TYPE_SIZE; i++)
@@ -1627,8 +1689,9 @@ main (int argc, char **argv)
 	      address_pad_len = 0;
 	      break;
 	    default:
-	      error (2, 0,
-		     _("invalid output address radix `%c'; it must be one character from [doxn]"),
+	      error (EXIT_FAILURE, 0,
+		     _("invalid output address radix `%c'; \
+it must be one character from [doxn]"),
 		     optarg[0]);
 	      break;
 	    }
@@ -1652,7 +1715,8 @@ main (int argc, char **argv)
 	    STRTOL_FATAL_ERROR (optarg, _("limit argument"), s_err);
 
 	  if (tmp > OFF_T_MAX)
-	    error (2, 0, _("specified number of bytes `%s' is larger than \
+	    error (EXIT_FAILURE, 0,
+		   _("specified number of bytes `%s' is larger than \
 the maximum\nrepresentable value of type off_t"), optarg);
 	  break;
 
@@ -1670,7 +1734,7 @@ the maximum\nrepresentable value of type off_t"), optarg);
 
 	case 't':
 	  if (decode_format_string (optarg))
-	    error (2, 0, _("invalid type string `%s'"), optarg);
+	    ++n_failed_decodes;
 	  break;
 
 	case 'v':
@@ -1689,9 +1753,8 @@ the maximum\nrepresentable value of type off_t"), optarg);
 #define CASE_OLD_ARG(old_char,new_string)		\
 	case old_char:					\
 	  {						\
-	    int tmp;					\
-	    tmp = decode_format_string (new_string);	\
-	    assert (tmp == 0);				\
+	    if (decode_format_string (new_string))	\
+	      ++n_failed_decodes;			\
 	  }						\
 	  break
 
@@ -1728,17 +1791,21 @@ the maximum\nrepresentable value of type off_t"), optarg);
 	}
     }
 
+  if (n_failed_decodes > 0)
+    exit (EXIT_FAILURE);
+
   if (show_version)
     {
-      printf ("od - %s\n", version_string);
-      exit (0);
+      printf ("od - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
     usage (0);
 
   if (flag_dump_strings && n_specs > 0)
-    error (2, 0, _("no type may be specified when dumping strings"));
+    error (EXIT_FAILURE, 0,
+	   _("no type may be specified when dumping strings"));
 
   n_files = argc - optind;
 
@@ -1804,14 +1871,14 @@ the maximum\nrepresentable value of type off_t"), optarg);
 	  else
 	    {
 	      error (0, 0,
-	      _("in compatibility mode the last 2 arguments must be offsets"));
+	    _("in compatibility mode the last 2 arguments must be offsets"));
 	      usage (1);
 	    }
 	}
       else
 	{
 	  error (0, 0,
-	     _("in compatibility mode there may be no more than 3 arguments"));
+	   _("in compatibility mode there may be no more than 3 arguments"));
 	  usage (1);
 	}
 
@@ -1842,9 +1909,13 @@ the maximum\nrepresentable value of type off_t"), optarg);
 
   if (n_specs == 0)
     {
-      int d_err = decode_one_format ("o2", NULL, &(spec[0]));
+      if (decode_one_format ("o2", "o2", NULL, &(spec[0])))
+	{
+	  /* This happens on Cray systems that don't have a 2-byte
+	     integral type.  */
+	  exit (EXIT_FAILURE);
+	}
 
-      assert (d_err == 0);
       n_specs = 1;
     }
 
@@ -1901,10 +1972,10 @@ the maximum\nrepresentable value of type off_t"), optarg);
 cleanup:;
 
   if (have_read_stdin && fclose (stdin) == EOF)
-    error (2, errno, _("standard input"));
+    error (EXIT_FAILURE, errno, _("standard input"));
 
   if (fclose (stdout) == EOF)
-    error (2, errno, _("write error"));
+    error (EXIT_FAILURE, errno, _("write error"));
 
-  exit (err);
+  exit (err == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
