@@ -44,7 +44,6 @@
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
-#include "version.h"
 #include "error.h"
 
 char *xmalloc ();
@@ -207,7 +206,7 @@ paste_parallel (int nfiles, char **fnamptr)
 	{
 	  fileptr[files_open] = fopen (fnamptr[files_open], "r");
 	  if (fileptr[files_open] == NULL)
-	    error (1, errno, "%s", fnamptr[files_open]);
+	    error (EXIT_FAILURE, errno, "%s", fnamptr[files_open]);
 	  else if (fileno (fileptr[files_open]) == 0)
 	    opened_stdin = 1;
 	}
@@ -216,7 +215,7 @@ paste_parallel (int nfiles, char **fnamptr)
   fileptr[files_open] = ENDLIST;
 
   if (opened_stdin && have_read_stdin)
-    error (1, 0, _("standard input is closed"));
+    error (EXIT_FAILURE, 0, _("standard input is closed"));
 
   /* Read a line from each file and output it to stdout separated by a
      delimiter, until we go through the loop without successfully
@@ -427,16 +426,20 @@ With no FILE, or when FILE is -, read standard input.\n\
 \n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
-void
+int
 main (int argc, char **argv)
 {
   int optc, exit_status;
   char default_delims[2], zero_delims[3];
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   have_read_stdin = 0;
   serial_merge = 0;
   delims = default_delims;
@@ -469,8 +472,8 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("paste - %s\n", version_string);
-      exit (0);
+      printf ("paste - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
@@ -486,8 +489,8 @@ main (int argc, char **argv)
   else
     exit_status = paste_serial (argc - optind, &argv[optind]);
   if (have_read_stdin && fclose (stdin) == EOF)
-    error (1, errno, "-");
+    error (EXIT_FAILURE, errno, "-");
   if (ferror (stdout) || fclose (stdout) == EOF)
-    error (1, errno, _("write error"));
-  exit (exit_status);
+    error (EXIT_FAILURE, errno, _("write error"));
+  exit (exit_status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
