@@ -1,5 +1,5 @@
 /* pr -- convert text files for printing.
-   Copyright (C) 1988, 1991, 1995 Free Software Foundation, Inc.
+   Copyright (C) 1988, 1991, 1995, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -101,7 +101,6 @@
 #include <sys/types.h>
 #include <time.h>
 #include "system.h"
-#include "version.h"
 #include "error.h"
 
 char *xmalloc ();
@@ -458,7 +457,7 @@ cols_ready_to_print (void)
   return n;
 }
 
-void
+int
 main (int argc, char **argv)
 {
   int c;
@@ -467,6 +466,9 @@ main (int argc, char **argv)
   char **file_names;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
   n_files = 0;
   file_names = (argc > 1
@@ -614,19 +616,19 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("pr - %s\n", version_string);
-      exit (0);
+      printf ("pr - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
     usage (0);
 
   if (parallel_files && explicit_columns)
-    error (1, 0,
+    error (EXIT_FAILURE, 0,
   _("Cannot specify number of columns when printing in parallel."));
 
   if (parallel_files && print_across_flag)
-    error (1, 0,
+    error (EXIT_FAILURE, 0,
   _("Cannot specify both printing across and printing in parallel."));
 
   for ( ; optind < argc; optind++)
@@ -654,12 +656,12 @@ main (int argc, char **argv)
   cleanup ();
 
   if (have_read_stdin && fclose (stdin) == EOF)
-    error (1, errno, _("standard input"));
+    error (EXIT_FAILURE, errno, _("standard input"));
   if (ferror (stdout) || fclose (stdout) == EOF)
-    error (1, errno, _("write error"));
+    error (EXIT_FAILURE, errno, _("write error"));
   if (failed_opens > 0)
     exit(1);
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
 
 /* Parse options of the form -scNNN.
@@ -743,7 +745,7 @@ init_parameters (int number_of_files)
 		      (columns - 1) * chars_per_gutter) / columns;
 
   if (chars_per_column < 1)
-    error (1, 0, _("page width too narrow"));
+    error (EXIT_FAILURE, 0, _("page width too narrow"));
 
   if (numbered_lines)
     {
@@ -955,9 +957,9 @@ close_file (COLUMN *p)
   if (p->status == CLOSED)
     return;
   if (ferror (p->fp))
-    error (1, errno, "%s", p->name);
+    error (EXIT_FAILURE, errno, "%s", p->name);
   if (p->fp != stdin && fclose (p->fp) == EOF)
-    error (1, errno, "%s", p->name);
+    error (EXIT_FAILURE, errno, "%s", p->name);
 
   if (!parallel_files)
     {
@@ -1882,5 +1884,5 @@ Paginate or columnate FILE(s) for printing.\n\
 spaces.  With no FILE, or when FILE is -, read standard input.\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
