@@ -1,5 +1,5 @@
 /* sort - sort lines of text (with all kinds of options).
-   Copyright (C) 1988, 1991, 1992, 1993, 1994, 1995 Free Software Foundation
+   Copyright (C) 88, 91, 92, 93, 94, 95, 1996 Free Software Foundation
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,8 +27,9 @@
 #include <sys/types.h>
 #include <signal.h>
 #include <stdio.h>
+#define NDEBUG 1
+#include <assert.h>
 #include "system.h"
-#include "version.h"
 #include "long-options.h"
 #include "error.h"
 #include "xstrtod.h"
@@ -57,9 +58,19 @@ void free ();
 #define DEFAULT_TMPDIR "/tmp"
 #endif
 
+/* Use this as exit status in case of error, not EXIT_FAILURE.  This
+   is necessary because EXIT_FAILURE is usually 1 and POSIX requires
+   that sort exit with status 1 IFF invoked with -c and the input is
+   not properly sorted.  Any other irregular exit must exit with a
+   status code greater than 1.  */
+#define SORT_FAILURE 2
+
 /* The kind of blanks for '-b' to skip in various options. */
 enum blanktype { bl_start, bl_end, bl_both };
 
+/* The character marking end of line. Default to \n. */
+int eolchar = '\n';
+
 /* Lines are held in core as counted strings. */
 struct line
 {
@@ -225,6 +236,7 @@ Write sorted concatenation of all FILE(s) to standard output.\n\
   -t SEP           use SEParator instead of non- to whitespace transition\n\
   -u               with -c, check for strict ordering\n\
   -u               with -m, only output the first of an equal sequence\n\
+  -z               end lines with 0 byte, not newline, for find -print0\n\
       --help       display this help and exit\n\
       --version    output version information and exit\n\
 \n\
@@ -236,6 +248,10 @@ FILE, or when FILE is -, read standard input.\n\
 ")
 	      , DEFAULT_TMPDIR);
     }
+  /* Don't use EXIT_FAILURE here in case it is defined to be 1.
+     POSIX requires that sort return 1 IFF invoked with -c and
+     the input is not properly sorted.  */
+  assert (status == 0 || status == SORT_FAILURE);
   exit (status);
 }
 
@@ -269,7 +285,7 @@ xmalloc (unsigned int n)
     {
       error (0, 0, _("virtual memory exhausted"));
       cleanup ();
-      exit (2);
+      exit (SORT_FAILURE);
     }
   return p;
 }
@@ -294,7 +310,7 @@ xrealloc (char *p, unsigned int n)
     {
       error (0, 0, _("virtual memory exhausted"));
       cleanup ();
-      exit (2);
+      exit (SORT_FAILURE);
     }
   return p;
 }
@@ -310,7 +326,7 @@ xtmpfopen (const char *file)
     {
       error (0, errno, "%s", file);
       cleanup ();
-      exit (2);
+      exit (SORT_FAILURE);
     }
 
   return fp;
@@ -331,7 +347,7 @@ xfopen (const char *file, const char *how)
 	{
 	  error (0, errno, "%s", file);
 	  cleanup ();
-	  exit (2);
+	  exit (SORT_FAILURE);
 	}
     }
 
@@ -355,7 +371,7 @@ xfclose (FILE *fp)
 	{
 	  error (0, errno, _("flushing file"));
 	  cleanup ();
-	  exit (2);
+	  exit (SORT_FAILURE);
 	}
     }
   else
@@ -364,19 +380,19 @@ xfclose (FILE *fp)
 	{
 	  error (0, errno, _("error closing file"));
 	  cleanup ();
-	  exit (2);
+	  exit (SORT_FAILURE);
 	}
     }
 }
 
 static void
-xfwrite (const char *buf, int size, int nelem, FILE *fp)
+write_bytes (const char *buf, size_t n_bytes, FILE *fp)
 {
-  if (fwrite (buf, size, nelem, fp) != nelem)
+  if (fwrite (buf, 1, n_bytes, fp) != n_bytes)
     {
       error (0, errno, _("write error"));
       cleanup ();
-      exit (2);
+      exit (SORT_FAILURE);
     }
 }
 
@@ -476,7 +492,7 @@ fillbuf (struct buffer *buf, FILE *fp)
   memmove (buf->buf, buf->buf + buf->used - buf->left, buf->left);
   buf->used = buf->left;
 
-  while (!feof (fp) && (buf->used == 0 || !memchr (buf->buf, '\n', buf->used)))
+  while (!feof (fp) && (buf->used == 0 || !memchr (buf->buf, eolchar, buf->used)))
     {
       if (buf->used == buf->alloc)
 	{
@@ -488,19 +504,19 @@ fillbuf (struct buffer *buf, FILE *fp)
 	{
 	  error (0, errno, _("read error"));
 	  cleanup ();
-	  exit (2);
+	  exit (SORT_FAILURE);
 	}
       buf->used += cc;
     }
 
-  if (feof (fp) && buf->used && buf->buf[buf->used - 1] != '\n')
+  if (feof (fp) && buf->used && buf->buf[buf->used - 1] != eolchar)
     {
       if (buf->used == buf->alloc)
 	{
 	  buf->alloc *= 2;
 	  buf->buf = xrealloc (buf->buf, buf->alloc);
 	}
-      buf->buf[buf->used++] = '\n';
+      buf->buf[buf->used++] = eolchar;
     }
 
   return buf->used;
@@ -594,6 +610,36 @@ limfield (const struct line *line, const struct keyfield *key)
 	  ++ptr;
       }
 
+#ifdef POSIX_UNSPECIFIED
+  /* The following block of code makes GNU sort incompatible with
+     standard Unix sort, so it's ifdef'd out for now.
+     The POSIX spec isn't clear on how to interpret this.
+     FIXME: request clarification.
+
+     From: kwzh@gnu.ai.mit.edu (Karl Heuer)
+     Date: Thu, 30 May 96 12:20:41 -0400
+
+     [...]I believe I've found another bug in `sort'.
+
+     $ cat /tmp/sort.in
+     a b c 2 d
+     pq rs 1 t
+     $ textutils-1.15/src/sort +0.6 -0.7 </tmp/sort.in
+     a b c 2 d
+     pq rs 1 t
+     $ /bin/sort +0.6 -0.7 </tmp/sort.in
+     pq rs 1 t
+     a b c 2 d
+
+     Unix sort produced the answer I expected: sort on the single character
+     in column 6.  GNU sort produced different results, because it disagrees
+     on the interpretation of the key-end spec "-M.N".  Unix sort reads this
+     as "skip M fields, then N characters"; but GNU sort wants it to mean
+     "skip M fields, then either N characters or the rest of the current
+     field, whichever comes first".  This extra clause applies only to
+     key-ends, not key-starts.
+     */
+
   /* Make LIM point to the end of (one byte past) the current field.  */
   if (tab)
     {
@@ -612,6 +658,7 @@ limfield (const struct line *line, const struct keyfield *key)
 	++newlim;
       lim = newlim;
     }
+#endif
 
   /* If we're skipping leading blanks, don't start counting characters
      until after skipping past any leading blanks.  */
@@ -648,7 +695,7 @@ findlines (struct buffer *buf, struct lines *lines)
 
   lines->used = 0;
 
-  while (beg < lim && (ptr = memchr (beg, '\n', lim - beg))
+  while (beg < lim && (ptr = memchr (beg, eolchar, lim - beg))
 	 && lines->used < lines->limit)
     {
       /* There are various places in the code that rely on a NUL
@@ -1303,8 +1350,8 @@ mergefps (FILE **fps, register int nfps, FILE *ofp)
 	{
 	  if (savedflag && compare (&saved, &lines[ord[0]].lines[cur[ord[0]]]))
 	    {
-	      xfwrite (saved.text, 1, saved.length, ofp);
-	      putc ('\n', ofp);
+	      write_bytes (saved.text, saved.length, ofp);
+	      putc (eolchar, ofp);
 	      savedflag = 0;
 	    }
 	  if (!savedflag)
@@ -1335,9 +1382,9 @@ mergefps (FILE **fps, register int nfps, FILE *ofp)
 	}
       else
 	{
-	  xfwrite (lines[ord[0]].lines[cur[ord[0]]].text, 1,
-		   lines[ord[0]].lines[cur[ord[0]]].length, ofp);
-	  putc ('\n', ofp);
+	  write_bytes (lines[ord[0]].lines[cur[ord[0]]].text,
+		       lines[ord[0]].lines[cur[ord[0]]].length, ofp);
+	  putc (eolchar, ofp);
 	}
 
       /* Check if we need to read more lines into core. */
@@ -1389,8 +1436,8 @@ mergefps (FILE **fps, register int nfps, FILE *ofp)
 
   if (unique && savedflag)
     {
-      xfwrite (saved.text, 1, saved.length, ofp);
-      putc ('\n', ofp);
+      write_bytes (saved.text, saved.length, ofp);
+      putc (eolchar, ofp);
       free (saved.text);
     }
 }
@@ -1542,8 +1589,8 @@ sort (char **files, int nfiles, FILE *ofp)
 	    if (!unique || i == 0
 		|| compare (&lines.lines[i], &lines.lines[i - 1]))
 	      {
-		xfwrite (lines.lines[i].text, 1, lines.lines[i].length, tfp);
-		putc ('\n', tfp);
+		write_bytes (lines.lines[i].text, lines.lines[i].length, tfp);
+		putc (eolchar, tfp);
 	      }
 	  if (tfp != ofp)
 	    xfclose (tfp);
@@ -1582,7 +1629,7 @@ insertkey (struct keyfield *key)
 static void
 badfieldspec (const char *s)
 {
-  error (2, 0, _("invalid field specification `%s'"), s);
+  error (SORT_FAILURE, 0, _("invalid field specification `%s'"), s);
 }
 
 /* Handle interrupts and hangups. */
@@ -1656,7 +1703,7 @@ set_ordering (register const char *s, struct keyfield *key,
   return (char *) s;
 }
 
-void
+int
 main (int argc, char **argv)
 {
   struct keyfield *key = NULL, gkey;
@@ -1670,8 +1717,11 @@ main (int argc, char **argv)
 #endif				/* SA_INTERRUPT */
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
-  parse_long_options (argc, argv, "sort", version_string, usage);
+  parse_long_options (argc, argv, "sort", PACKAGE_VERSION, usage);
 
   have_read_stdin = 0;
   inittables ();
@@ -1754,7 +1804,12 @@ main (int argc, char **argv)
 	  if (digits[UCHAR (*s)] || (*s == '.' && digits[UCHAR (s[1])]))
 	    {
 	      if (!key)
-		usage (2);
+		{
+		  /* Provoke with `sort -9'.  */
+		  error (0, 0, _("when using the old-style +POS and -POS \
+key specifiers,\nthe +POS specifier must come first"));
+		  usage (SORT_FAILURE);
+		}
 	      for (t = 0; digits[UCHAR (*s)]; ++s)
 		t = t * 10 + *s - '0';
 	      t2 = 0;
@@ -1786,7 +1841,8 @@ main (int argc, char **argv)
 		    else
 		      {
 			if (i == argc - 1)
-			  error (2, 0, _("option `-k' requires an argument"));
+			  error (SORT_FAILURE, 0,
+				 _("option `-k' requires an argument"));
 			else
 			  s = argv[++i];
 		      }
@@ -1906,7 +1962,8 @@ but lacks following character offset"));
 		    else
 		      {
 			if (i == argc - 1)
-			  error (2, 0, _("option `-o' requires an argument"));
+			  error (SORT_FAILURE, 0,
+				 _("option `-o' requires an argument"));
 			else
 			  outfile = argv[++i];
 		      }
@@ -1923,7 +1980,8 @@ but lacks following character offset"));
 			goto outer;
 		      }
 		    else
-		      error (2, 0, _("option `-t' requires an argument"));
+		      error (SORT_FAILURE, 0,
+			     _("option `-t' requires an argument"));
 		    break;
 		  case 'T':
 		    if (s[1])
@@ -1933,13 +1991,17 @@ but lacks following character offset"));
 			if (i < argc - 1)
 			  temp_file_prefix = argv[++i];
 			else
-			  error (2, 0, _("option `-T' requires an argument"));
+			  error (SORT_FAILURE, 0,
+				 _("option `-T' requires an argument"));
 		      }
 		    goto outer;
 		    /* break; */
 		  case 'u':
 		    unique = 1;
 		    break;
+		  case 'z':
+		    eolchar = 0;
+		    break;
 		  case 'y':
 		    /* Accept and ignore e.g. -y0 for compatibility with
 		       Solaris 2.  */
@@ -1947,7 +2009,7 @@ but lacks following character offset"));
 		  default:
 		    fprintf (stderr, _("%s: unrecognized option `-%c'\n"),
 			     argv[0], *s);
-		    usage (2);
+		    usage (SORT_FAILURE);
 		  }
 		if (*s)
 		  ++s;
@@ -1992,7 +2054,11 @@ but lacks following character offset"));
     }
 
   if (checkonly)
-    exit (check (files, nfiles) != 0);
+    {
+      /* POSIX requires that sort return 1 IFF invoked with -c and the
+	 input is not properly sorted.  */
+      exit (check (files, nfiles) == 0 ? 0 : 1);
+    }
 
   if (strcmp (outfile, "-"))
     {
@@ -2019,11 +2085,11 @@ but lacks following character offset"));
 		  struct stat instat;
 		  if ((strcmp (files[i], "-")
 		       ? stat (files[i], &instat)
-		       : fstat (fileno (stdin), &instat)) != 0)
+		       : fstat (STDIN_FILENO, &instat)) != 0)
 		    {
 		      error (0, errno, "%s", files[i]);
 		      cleanup ();
-		      exit (2);
+		      exit (SORT_FAILURE);
 		    }
 		  if (S_ISREG (instat.st_mode)
 		      && (instat.st_ino != outstat.st_ino
@@ -2038,12 +2104,12 @@ but lacks following character offset"));
 	      tmp = tempname ();
 	      ofp = xtmpfopen (tmp);
 	      while ((cc = fread (buf, 1, sizeof buf, fp)) > 0)
-		xfwrite (buf, 1, cc, ofp);
+		write_bytes (buf, cc, ofp);
 	      if (ferror (fp))
 		{
 		  error (0, errno, "%s", files[i]);
 		  cleanup ();
-		  exit (2);
+		  exit (SORT_FAILURE);
 		}
 	      xfclose (ofp);
 	      xfclose (fp);
@@ -2067,12 +2133,12 @@ but lacks following character offset"));
      Solaris, Ultrix, and Irix.  This premature fflush makes the output
      reappear. --karl@cs.umb.edu  */
   if (fflush (ofp) < 0)
-    error (1, errno, _("%s: write error"), outfile);
+    error (SORT_FAILURE, errno, _("%s: write error"), outfile);
 
   if (have_read_stdin && fclose (stdin) == EOF)
-    error (1, errno, outfile);
+    error (SORT_FAILURE, errno, outfile);
   if (ferror (stdout) || fclose (stdout) == EOF)
-    error (1, errno, _("%s: write error"), outfile);
+    error (SORT_FAILURE, errno, _("%s: write error"), outfile);
 
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
