@@ -1,5 +1,5 @@
 /* split.c -- split a file into pieces.
-   Copyright (C) 1988, 1991, 1995 Free Software Foundation, Inc.
+   Copyright (C) 88, 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -40,7 +40,6 @@
 #endif
 
 #include "system.h"
-#include "version.h"
 #include "error.h"
 #include "xstrtol.h"
 
@@ -122,7 +121,7 @@ PREFIX is `x'.  With no INPUT, or when INPUT is -, read standard input.\n\
 SIZE may have a multiplier suffix: b for 512, k for 1K, m for 1 Meg.\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 /* Compute the next sequential output file name suffix and store it
@@ -177,17 +176,17 @@ cwrite (int new_file_flag, const char *bp, int bytes)
   if (new_file_flag)
     {
       if (output_desc >= 0 && close (output_desc) < 0)
-	error (1, errno, "%s", outfile);
+	error (EXIT_FAILURE, errno, "%s", outfile);
 
       next_file_name ();
       if (verbose)
 	fprintf (stderr, _("creating file `%s'\n"), outfile);
       output_desc = open (outfile, O_WRONLY | O_CREAT | O_TRUNC, 0666);
       if (output_desc < 0)
-	error (1, errno, "%s", outfile);
+	error (EXIT_FAILURE, errno, "%s", outfile);
     }
   if (full_write (output_desc, bp, bytes) < 0)
-    error (1, errno, "%s", outfile);
+    error (EXIT_FAILURE, errno, "%s", outfile);
 }
 
 /* Read NCHARS bytes from the input file into BUF.
@@ -229,7 +228,7 @@ bytes_split (int nchars, char *buf, int bufsize)
     {
       n_read = stdread (buf, bufsize);
       if (n_read < 0)
-        error (1, errno, "%s", infile);
+        error (EXIT_FAILURE, errno, "%s", infile);
       bp_out = buf;
       to_read = n_read;
       for (;;)
@@ -272,7 +271,7 @@ lines_split (int nlines, char *buf, int bufsize)
     {
       n_read = stdread (buf, bufsize);
       if (n_read < 0)
-	error (1, errno, "%s", infile);
+	error (EXIT_FAILURE, errno, "%s", infile);
       bp = bp_out = buf;
       eob = bp + n_read;
       *eob = '\n';
@@ -321,7 +320,7 @@ line_bytes_split (int nchars)
 
       n_read = stdread (buf + n_buffered, nchars - n_buffered);
       if (n_read < 0)
-	error (1, errno, "%s", infile);
+	error (EXIT_FAILURE, errno, "%s", infile);
 
       n_buffered += n_read;
       if (n_buffered != nchars)
@@ -353,7 +352,7 @@ line_bytes_split (int nchars)
   free (buf);
 }
 
-void
+int
 main (int argc, char **argv)
 {
   struct stat stat_buf;
@@ -370,6 +369,9 @@ main (int argc, char **argv)
   int digits_optind = 0;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
   /* Parse command line options.  */
 
@@ -451,8 +453,8 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("split - %s\n", version_string);
-      exit (0);
+      printf ("split - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
@@ -487,7 +489,7 @@ main (int argc, char **argv)
     {
       input_desc = open (infile, O_RDONLY);
       if (input_desc < 0)
-	error (1, errno, "%s", infile);
+	error (EXIT_FAILURE, errno, "%s", infile);
     }
 
   /* No output file is open now.  */
@@ -507,7 +509,7 @@ main (int argc, char **argv)
   /* Get the optimal block size of input device and make a buffer.  */
 
   if (fstat (input_desc, &stat_buf) < 0)
-    error (1, errno, "%s", infile);
+    error (EXIT_FAILURE, errno, "%s", infile);
   in_blk_size = ST_BLKSIZE (stat_buf);
 
   buf = xmalloc (in_blk_size + 1);
@@ -532,9 +534,9 @@ main (int argc, char **argv)
     }
 
   if (close (input_desc) < 0)
-    error (1, errno, "%s", infile);
+    error (EXIT_FAILURE, errno, "%s", infile);
   if (output_desc >= 0 && close (output_desc) < 0)
-    error (1, errno, "%s", outfile);
+    error (EXIT_FAILURE, errno, "%s", outfile);
 
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
