@@ -1,5 +1,5 @@
 /* sum -- checksum and count the blocks in a file
-   Copyright (C) 1986, 1989, 1991, 1995 Free Software Foundation, Inc.
+   Copyright (C) 86, 89, 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 #include <sys/types.h>
 #include <getopt.h>
 #include "system.h"
-#include "version.h"
 #include "error.h"
 
 int safe_read ();
@@ -76,7 +75,7 @@ Print checksum and block counts for each FILE.\n\
 With no FILE, or when FILE is -, read standard input.\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 /* Calculate and print the rotated checksum and the size in 1K blocks
@@ -198,7 +197,7 @@ sysv_sum_file (const char *file, int print_name)
   return 0;
 }
 
-void
+int
 main (int argc, char **argv)
 {
   int errors = 0;
@@ -207,6 +206,10 @@ main (int argc, char **argv)
   int (*sum_func) () = bsd_sum_file;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   have_read_stdin = 0;
 
   while ((optc = getopt_long (argc, argv, "rs", longopts, (int *) 0)) != -1)
@@ -231,8 +234,8 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("sum - %s\n", version_string);
-      exit (0);
+      printf ("sum - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
@@ -250,7 +253,7 @@ main (int argc, char **argv)
 	errors = 1;
 
   if (have_read_stdin && fclose (stdin) == EOF)
-    error (1, errno, "-");
-  exit (errors);
+    error (EXIT_FAILURE, errno, "-");
+  exit (errors == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
