@@ -1,5 +1,5 @@
 /* tac - concatenate and print files in reverse
-   Copyright (C) 1988, 1989, 1990, 1991, 1995 Free Software Foundation, Inc.
+   Copyright (C) 88, 89, 90, 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -47,7 +47,6 @@ tac -r -s '.\|
 # include <rx.h>
 #endif
 #include "system.h"
-#include "version.h"
 #include "error.h"
 
 #ifndef STDC_HEADERS
@@ -146,7 +145,7 @@ With no FILE, or when FILE is -, read standard input.\n\
       --version            output version information and exit\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 static void
@@ -159,7 +158,7 @@ static void
 cleanup_fatal (void)
 {
   cleanup ();
-  exit (1);
+  exit (EXIT_FAILURE);
 }
 
 static RETSIGTYPE
@@ -580,7 +579,7 @@ tac_stdin (void)
   return errors;
 }
 
-void
+int
 main (int argc, char **argv)
 {
   const char *error_message;	/* Return value from re_compile_pattern. */
@@ -588,6 +587,10 @@ main (int argc, char **argv)
   int have_read_stdin = 0;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   errors = 0;
   separator = "\n";
   sentinel_length = 1;
@@ -609,7 +612,7 @@ main (int argc, char **argv)
 	case 's':
 	  separator = optarg;
 	  if (*separator == 0)
-	    error (1, 0, _("separator cannot be empty"));
+	    error (EXIT_FAILURE, 0, _("separator cannot be empty"));
 	  break;
 	default:
 	  usage (1);
@@ -618,8 +621,8 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("tac - %s\n", version_string);
-      exit (0);
+      printf ("tac - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
@@ -635,7 +638,7 @@ main (int argc, char **argv)
       error_message = re_compile_pattern (separator, strlen (separator),
 					  &compiled_separator);
       if (error_message)
-	error (1, 0, "%s", error_message);
+	error (EXIT_FAILURE, 0, "%s", error_message);
     }
   else
     match_length = sentinel_length = strlen (separator);
@@ -675,8 +678,8 @@ main (int argc, char **argv)
   output ((char *) NULL, (char *) NULL);
 
   if (have_read_stdin && close (0) < 0)
-    error (1, errno, "-");
+    error (EXIT_FAILURE, errno, "-");
   if (close (1) < 0)
-    error (1, errno, _("write error"));
-  exit (errors);
+    error (EXIT_FAILURE, errno, _("write error"));
+  exit (errors == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
