@@ -1,5 +1,5 @@
 /* tr -- a filter to translate characters
-   Copyright (C) 1991, 1995 Free Software Foundation, Inc.
+   Copyright (C) 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,7 +34,6 @@
 #endif
 
 #include "system.h"
-#include "version.h"
 #include "error.h"
 
 #ifndef ULONG_MAX
@@ -205,9 +204,9 @@ struct Spec_list
        class construct e.g. [=c=].  */
     int has_equiv_class;
 
-    /* Non-zero if this spec contains at least one of [:upper:] or
-       [:lower:] class constructs.  */
-    int has_upper_or_lower;
+    /* Non-zero if this spec contains at least one character class
+       construct.  E.g. [:digit:].  */
+    int has_char_class;
 
     /* Non-zero if this spec contains at least one of the character class
        constructs (all but upper and lower) that aren't allowed in s2.  */
@@ -414,7 +413,7 @@ translating nor deleting; else squeezing uses SET2 and occurs after\n\
 translation or deletion.\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 /* Return nonzero if the character C is a member of the
@@ -966,9 +965,9 @@ find_bracketed_repeat (const struct E_string *es, size_t start_idx,
    the `]' must not be escaped.  */
 
 static int
-star_digits_closebracket (const struct E_string *es, int idx)
+star_digits_closebracket (const struct E_string *es, size_t idx)
 {
-  int i;
+  size_t i;
 
   if (!ES_MATCH (es, idx, '*'))
     return 0;
@@ -1334,7 +1333,7 @@ get_spec_stats (struct Spec_list *s)
   s->n_indefinite_repeats = 0;
   s->has_equiv_class = 0;
   s->has_restricted_char_class = 0;
-  s->has_upper_or_lower = 0;
+  s->has_char_class = 0;
   for (p = s->head->next; p; p = p->next)
     {
       switch (p->type)
@@ -1350,6 +1349,7 @@ get_spec_stats (struct Spec_list *s)
 	  break;
 
 	case RE_CHAR_CLASS:
+	  s->has_char_class = 1;
 	  for (i = 0; i < N_CHARS; i++)
 	    if (is_char_class_member (p->u.char_class, i))
 	      ++len;
@@ -1357,7 +1357,6 @@ get_spec_stats (struct Spec_list *s)
 	    {
 	    case CC_UPPER:
 	    case CC_LOWER:
-	      s->has_upper_or_lower = 1;
 	      break;
 	    default:
 	      s->has_restricted_char_class = 1;
@@ -1503,6 +1502,27 @@ string2_extend (const struct Spec_list *s1, struct Spec_list *s2)
   s2->length = s1->length;
 }
 
+/* Return non-zero if S is a non-empty list in which exactly one
+   character (but potentially, many instances of it) appears.
+   E.g.  [X*] or xxxxxxxx.  */
+
+static int
+homogeneous_spec_list (struct Spec_list *s)
+{
+  int b, c;
+
+  s->state = BEGIN_STATE;
+
+  if ((b = get_next (s, NULL)) == -1)
+    return 0;
+
+  while ((c = get_next (s, NULL)) != -1)
+    if (c != b)
+      return 0;
+
+  return 1;
+}
+
 /* Die with an error message if S1 and S2 describe strings that
    are not valid with the given command line switches.
    A side effect of this function is that if a valid [c*] or
@@ -1517,32 +1537,17 @@ validate (struct Spec_list *s1, struct Spec_list *s2)
   get_s1_spec_stats (s1);
   if (s1->n_indefinite_repeats > 0)
     {
-      error (1, 0, _("the [c*] repeat construct may not appear in string1"));
-    }
-
-  /* FIXME: it isn't clear from the POSIX spec that this is invalid,
-     but in the spirit of the other restrictions put on translation
-     with character classes, this seems a logical interpretation.  */
-  if (complement && s1->has_upper_or_lower)
-    {
-      error (1, 0,
-	     _("character classes may not be used when translating \
-and complementing"));
+      error (EXIT_FAILURE, 0,
+	     _("the [c*] repeat construct may not appear in string1"));
     }
 
   if (s2)
     {
       get_s2_spec_stats (s2, s1->length);
-      if (s2->has_restricted_char_class)
-	{
-	  error (1, 0,
-		 _("when translating, the only character classes that may \
-appear in\n\tstring2 are `upper' and `lower'"));
-	}
 
       if (s2->n_indefinite_repeats > 1)
 	{
-	  error (1, 0,
+	  error (EXIT_FAILURE, 0,
 		 _("only one [c*] repeat construct may appear in string2"));
 	}
 
@@ -1550,7 +1555,7 @@ appear in\n\tstring2 are `upper' and `lower'"));
 	{
 	  if (s2->has_equiv_class)
 	    {
-	      error (1, 0,
+	      error (EXIT_FAILURE, 0,
 		     _("[=c=] expressions may not appear in string2 \
 when translating"));
 	    }
@@ -1563,22 +1568,32 @@ when translating"));
 		     given or string1 is empty.  */
 
 		  if (s2->length == 0)
-		    error (1, 0,
+		    error (EXIT_FAILURE, 0,
 		     _("when not truncating set1, string2 must be non-empty"));
 		  string2_extend (s1, s2);
 		}
 	    }
 
-	  if (complement && s2->has_upper_or_lower)
-	    error (1, 0,
-		   _("character classes may not be used when translating \
-and complementing"));
+	  if (complement && s1->has_char_class
+	      && ! (s2->length == s1->length && homogeneous_spec_list (s2)))
+	    {
+	      error (EXIT_FAILURE, 0,
+		     _("when translating with complemented character classes,\
+\nstring2 must map all characters in the domain to one"));
+	    }
+
+	  if (s2->has_restricted_char_class)
+	    {
+	      error (EXIT_FAILURE, 0,
+		     _("when translating, the only character classes that may \
+appear in\n\tstring2 are `upper' and `lower'"));
+	    }
 	}
       else
 	/* Not translating.  */
 	{
 	  if (s2->n_indefinite_repeats > 0)
-	    error (1, 0,
+	    error (EXIT_FAILURE, 0,
 		   _("the [c*] construct may appear in string2 only \
 when translating"));
 	}
@@ -1612,7 +1627,7 @@ squeeze_filter (unsigned char *buf, long int size, PFI reader)
 	    nr = (*reader) (buf, size, NULL);
 
 	  if (nr < 0)
-	    error (1, errno, _("read error"));
+	    error (EXIT_FAILURE, errno, _("read error"));
 	  if (nr == 0)
 	    break;
 	  i = 0;
@@ -1663,7 +1678,7 @@ squeeze_filter (unsigned char *buf, long int size, PFI reader)
 	    }
 	  if (out_len > 0
 	      && fwrite ((char *) &buf[begin], 1, out_len, stdout) == 0)
-	    error (1, errno, _("write error"));
+	    error (EXIT_FAILURE, errno, _("write error"));
 	}
 
       if (char_to_squeeze != NOT_A_CHAR)
@@ -1709,7 +1724,7 @@ read_and_delete (unsigned char *buf, long int size, PFI not_used)
       int nr = safe_read (0, (char *) buf, size);
 
       if (nr < 0)
-	error (1, errno, _("read error"));
+	error (EXIT_FAILURE, errno, _("read error"));
       if (nr == 0)
 	{
 	  hit_eof = 1;
@@ -1753,7 +1768,7 @@ read_and_xlate (unsigned char *buf, long int size, PFI not_used)
 
   chars_read = safe_read (0, (char *) buf, size);
   if (chars_read < 0)
-    error (1, errno, _("read error"));
+    error (EXIT_FAILURE, errno, _("read error"));
   if (chars_read == 0)
     {
       hit_eof = 1;
@@ -1786,7 +1801,7 @@ set_initialize (struct Spec_list *s, int complement_this_set, SET_TYPE *in_set)
       in_set[i] = (!in_set[i]);
 }
 
-void
+int
 main (int argc, char **argv)
 {
   int c;
@@ -1796,6 +1811,9 @@ main (int argc, char **argv)
   struct Spec_list *s2 = &buf2;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
   while ((c = getopt_long (argc, argv, "cdst", long_options,
 			   (int *) 0)) != EOF)
@@ -1829,8 +1847,8 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("tr - %s\n", version_string);
-      exit (0);
+      printf ("tr - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
@@ -1853,10 +1871,10 @@ main (int argc, char **argv)
     }
 
   if (!delete && !squeeze_repeats && non_option_args != 2)
-    error (1, 0, _("two strings must be given when translating"));
+    error (EXIT_FAILURE, 0, _("two strings must be given when translating"));
 
   if (delete && squeeze_repeats && non_option_args != 2)
-    error (1, 0, _("two strings must be given when both \
+    error (EXIT_FAILURE, 0, _("two strings must be given when both \
 deleting and squeezing repeats"));
 
   /* If --delete is given without --squeeze-repeats, then
@@ -1869,24 +1887,24 @@ deleting and squeezing repeats"));
       if (posix_pedantic && non_option_args == 2)
 	--non_option_args;
       else
-	error (1, 0,
+	error (EXIT_FAILURE, 0,
 	       _("only one string may be given when deleting \
 without squeezing repeats"));
     }
 
   if (squeeze_repeats && non_option_args == 0)
-    error (1, 0,
+    error (EXIT_FAILURE, 0,
 	   _("at least one string must be given when squeezing repeats"));
 
   spec_init (s1);
   if (parse_str ((unsigned char *) argv[optind], s1))
-    exit (1);
+    exit (EXIT_FAILURE);
 
   if (non_option_args == 2)
     {
       spec_init (s2);
       if (parse_str ((unsigned char *) argv[optind + 1], s2))
-	exit (1);
+	exit (EXIT_FAILURE);
     }
   else
     s2 = NULL;
@@ -1907,7 +1925,7 @@ without squeezing repeats"));
 	{
 	  nr = read_and_delete (io_buf, IO_BUF_SIZE, NULL);
 	  if (nr > 0 && fwrite ((char *) io_buf, 1, nr, stdout) == 0)
-	    error (1, errno, _("write error"));
+	    error (EXIT_FAILURE, errno, _("write error"));
 	}
       while (nr > 0);
     }
@@ -1961,7 +1979,7 @@ without squeezing repeats"));
 	      c1 = get_next (s1, &class_s1);
 	      c2 = get_next (s2, &class_s2);
 	      if (!class_ok[(int) class_s1][(int) class_s2])
-		error (1, 0,
+		error (EXIT_FAILURE, 0,
 		     _("misaligned or mismatched upper and/or lower classes"));
 	      /* The following should have been checked by validate...  */
 	      if (c2 == -1)
@@ -1984,17 +2002,17 @@ without squeezing repeats"));
 	      chars_read = read_and_xlate (io_buf, IO_BUF_SIZE, NULL);
 	      if (chars_read > 0
 		  && fwrite ((char *) io_buf, 1, chars_read, stdout) == 0)
-		error (1, errno, _("write error"));
+		error (EXIT_FAILURE, errno, _("write error"));
 	    }
 	  while (chars_read > 0);
 	}
     }
 
   if (fclose (stdout) == EOF)
-    error (2, errno, _("write error"));
+    error (EXIT_FAILURE, errno, _("write error"));
 
   if (close (0) != 0)
-    error (2, errno, _("standard input"));
+    error (EXIT_FAILURE, errno, _("standard input"));
 
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
