@@ -1,5 +1,5 @@
 /* uniq -- remove duplicate lines from a sorted file
-   Copyright (C) 1986, 1991, 1995 Free Software Foundation, Inc.
+   Copyright (C) 86, 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -40,9 +40,9 @@
 
 #include "system.h"
 #include "linebuffer.h"
-#include "version.h"
 #include "error.h"
 #include "xstrtol.h"
+#include "memcasecmp.h"
 
 /* Undefine, to avoid warning about redefinition on some systems.  */
 #undef min
@@ -80,6 +80,9 @@ enum output_mode
 /* Which lines to output. */
 static enum output_mode mode;
 
+/* If nonzero, ignore case when comparing.  */
+static int ignore_case;
+
 /* If nonzero, display usage information and exit.  */
 static int show_help;
 
@@ -90,6 +93,7 @@ static struct option const longopts[] =
 {
   {"count", no_argument, NULL, 'c'},
   {"repeated", no_argument, NULL, 'd'},
+  {"ignore-case", no_argument, NULL, 'i'},
   {"unique", no_argument, NULL, 'u'},
   {"skip-fields", required_argument, NULL, 'f'},
   {"skip-chars", required_argument, NULL, 's'},
@@ -118,6 +122,7 @@ standard input), writing to OUTPUT (or standard output).\n\
   -c, --count           prefix lines by the number of occurrences\n\
   -d, --repeated        only print duplicate lines\n\
   -f, --skip-fields=N   avoid comparing the first N fields\n\
+  -i, --ignore-case     ignore differences in case when comparing\n\
   -s, --skip-chars=N    avoid comparing the first N characters\n\
   -u, --unique          only print unique lines\n\
   -w, --check-chars=N   compare no more than N characters in lines\n\
@@ -127,10 +132,10 @@ standard input), writing to OUTPUT (or standard output).\n\
       --version         output version information and exit\n\
 \n\
 A field is a run of whitespace, than non-whitespace characters.\n\
-Fields are skipped before chars. \n\
+Fields are skipped before chars.\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 /* Given a linebuffer LINE,
@@ -175,12 +180,20 @@ different (const char *old, const char *new, int oldlen, int newlen)
       if (newlen > check_chars)
 	newlen = check_chars;
     }
-  order = memcmp (old, new, min (oldlen, newlen));
+
+  /* Use an if-statement here rather than a function variable to
+     avoid portability hassles of getting a non-conflicting declaration
+     of memcmp.  */
+  if (ignore_case)
+    order = memcasecmp (old, new, min (oldlen, newlen));
+  else
+    order = memcmp (old, new, min (oldlen, newlen));
+
   if (order == 0)
     return oldlen - newlen;
   return order;
 }
-
+
 /* Output the line in linebuffer LINE to stream STREAM
    provided that the switches say it should be output.
    If requested, print the number of times it occurred, as well;
@@ -219,14 +232,14 @@ check_file (const char *infile, const char *outfile)
   else
     istream = fopen (infile, "r");
   if (istream == NULL)
-    error (1, errno, "%s", infile);
+    error (EXIT_FAILURE, errno, "%s", infile);
 
   if (!strcmp (outfile, "-"))
     ostream = stdout;
   else
     ostream = fopen (outfile, "w");
   if (ostream == NULL)
-    error (1, errno, "%s", outfile);
+    error (EXIT_FAILURE, errno, "%s", outfile);
 
   thisline = &lb1;
   prevline = &lb2;
@@ -264,29 +277,33 @@ check_file (const char *infile, const char *outfile)
 
  closefiles:
   if (ferror (istream) || fclose (istream) == EOF)
-    error (1, errno, _("error reading %s"), infile);
+    error (EXIT_FAILURE, errno, _("error reading %s"), infile);
 
   if (ferror (ostream) || fclose (ostream) == EOF)
-    error (1, errno, _("error writing %s"), outfile);
+    error (EXIT_FAILURE, errno, _("error writing %s"), outfile);
 
   free (lb1.buffer);
   free (lb2.buffer);
 }
 
-void
+int
 main (int argc, char **argv)
 {
   int optc;
   char *infile = "-", *outfile = "-";
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   skip_chars = 0;
   skip_fields = 0;
   check_chars = 0;
   mode = output_all;
   countmode = count_none;
 
-  while ((optc = getopt_long (argc, argv, "0123456789cdf:s:uw:", longopts,
+  while ((optc = getopt_long (argc, argv, "0123456789cdf:is:uw:", longopts,
 			      (int *) 0)) != EOF)
     {
       switch (optc)
@@ -320,18 +337,24 @@ main (int argc, char **argv)
 	    long int tmp_long;
 	    if (xstrtol (optarg, NULL, 10, &tmp_long, NULL) != LONGINT_OK
 		|| tmp_long <= 0 || tmp_long > INT_MAX)
-	      error (1, 0, _("invalid number of fields to skip: `%s'"),
+	      error (EXIT_FAILURE, 0,
+		     _("invalid number of fields to skip: `%s'"),
 		     optarg);
 	    skip_fields = (int) tmp_long;
 	  }
 	  break;
 
+	case 'i':
+	  ignore_case = 1;
+	  break;
+
 	case 's':		/* Like '+#'. */
 	  {
 	    long int tmp_long;
 	    if (xstrtol (optarg, NULL, 10, &tmp_long, NULL) != LONGINT_OK
 		|| tmp_long <= 0 || tmp_long > INT_MAX)
-	      error (1, 0, _("invalid number of bytes to skip: `%s'"),
+	      error (EXIT_FAILURE, 0,
+		     _("invalid number of bytes to skip: `%s'"),
 		     optarg);
 	    skip_chars = (int) tmp_long;
 	  }
@@ -346,7 +369,8 @@ main (int argc, char **argv)
 	    long int tmp_long;
 	    if (xstrtol (optarg, NULL, 10, &tmp_long, NULL) != LONGINT_OK
 		|| tmp_long <= 0 || tmp_long > INT_MAX)
-	      error (1, 0, _("invalid number of bytes to compare: `%s'"),
+	      error (EXIT_FAILURE, 0,
+		     _("invalid number of bytes to compare: `%s'"),
 		     optarg);
 	    check_chars = (int) tmp_long;
 	  }
@@ -359,8 +383,8 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("uniq - %s\n", version_string);
-      exit (0);
+      printf ("uniq - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
@@ -376,7 +400,8 @@ main (int argc, char **argv)
 	  long int tmp_long;
 	  if (xstrtol (opt_str, NULL, 10, &tmp_long, NULL) != LONGINT_OK
 	      || tmp_long <= 0 || tmp_long > INT_MAX)
-	    error (1, 0, _("invalid number of bytes to compare: `%s'"),
+	    error (EXIT_FAILURE, 0,
+		   _("invalid number of bytes to compare: `%s'"),
 		   opt_str);
 	  skip_chars = (int) tmp_long;
 	}
@@ -393,5 +418,5 @@ main (int argc, char **argv)
 
   check_file (infile, outfile);
 
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
