@@ -1,5 +1,5 @@
 /* wc - print the number of bytes, words, and lines in files
-   Copyright (C) 1985, 1991, 1995 Free Software Foundation, Inc.
+   Copyright (C) 85, 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
-#include "version.h"
 #include "error.h"
 
 /* Size of atomic reads. */
@@ -87,7 +86,7 @@ read standard input.\n\
       --version          output version information and exit\n\
 "));
     }
-  exit (status);
+  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 static void
@@ -254,13 +253,17 @@ wc_file (const char *file)
     }
 }
 
-void
+int
 main (int argc, char **argv)
 {
   int optc;
   int nfiles;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   exit_status = 0;
   print_lines = print_words = print_chars = 0;
   total_lines = total_words = total_chars = 0;
@@ -289,8 +292,8 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("wc - %s\n", version_string);
-      exit (0);
+      printf ("wc - %s\n", PACKAGE_VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
@@ -316,7 +319,7 @@ main (int argc, char **argv)
     }
 
   if (have_read_stdin && close (0))
-    error (1, errno, "-");
+    error (EXIT_FAILURE, errno, "-");
 
-  exit (exit_status);
+  exit (exit_status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
